@@ -281,6 +281,10 @@ var FilterView =
     if (value == this._subscription)
       return;
 
+    // Make sure the editor is done before we update the list.
+    if (this.treeElement)
+      this.treeElement.stopEditing(true);
+
     this._subscription = value;
     this.refresh(true);
   },
@@ -612,7 +616,7 @@ var FilterView =
       this.editDummy = {filter: {text: ""}};
 
       let atomService = Cc["@mozilla.org/atom-service;1"].getService(Ci.nsIAtomService);
-      let stringAtoms = ["col-filter", "col-enabled", "col-hitcount", "col-lasthit", "type-comment", "type-filterlist", "type-whitelist", "type-elemhide", "type-invalid"];
+      let stringAtoms = ["col-filter", "col-enabled", "col-hitcount", "col-lasthit", "type-comment", "type-filterlist", "type-whitelist", "type-elemhide", "type-elemhideexception", "type-invalid"];
       let boolAtoms = ["selected", "dummy", "slow", "disabled"];
 
       this.atoms = {};
@@ -658,8 +662,8 @@ var FilterView =
       else if (col == "col-lasthit")
         return (filter.lastHit ? Utils.formatTime(filter.lastHit) : null);
     }
-    else
-      return null;
+
+    return null;
   },
 
   getColumnProperties: function(col, properties)
@@ -690,6 +694,8 @@ var FilterView =
       properties.AppendElement(this.atoms["type-whitelist"]);
     else if (filter instanceof ElemHideFilter)
       properties.AppendElement(this.atoms["type-elemhide"]);
+    else if (filter instanceof ElemHideException)
+      properties.AppendElement(this.atoms["type-elemhideexception"]);
     else if (filter instanceof InvalidFilter)
       properties.AppendElement(this.atoms["type-invalid"]);
   },
@@ -733,7 +739,7 @@ var FilterView =
 
     let item = this.data[row];
     let position = (orientation == Ci.nsITreeView.DROP_BEFORE ? item.index : item.index + 1);
-    return FilterActions.drop(position, dataTransfer);
+    FilterActions.drop(position, dataTransfer);
   },
 
   isEditable: function(row, col)
@@ -773,7 +779,7 @@ var FilterView =
   cycleCell: function(row, col)
   {
     if (row < 0 || row >= this.data.length || col.id != "col-enabled")
-      return null;
+      return;
 
     let filter = this.data[row].filter;
     if (filter instanceof ActiveFilter)
