@@ -8,16 +8,24 @@
 // Report data template, more data will be added during data collection
 //
 
+Cu.import("resource://gre/modules/Services.jsm");
+Cu.import("resource://gre/modules/FileUtils.jsm");
+
+const MILLISECONDS_IN_SECOND = 1000;
+const SECONDS_IN_MINUTE = 60;
+const SECONDS_IN_HOUR = 60 * SECONDS_IN_MINUTE;
+const SECONDS_IN_DAY = 24 * SECONDS_IN_HOUR;
+
 let reportData =
   <report>
-    <adblock-plus version={Utils.addonVersion} build={Utils.addonBuild} locale={Utils.appLocale}/>
-    <application name={Utils.appInfo.name} vendor={Utils.appInfo.vendor} version={Utils.appInfo.version} userAgent={window.navigator.userAgent}/>
-    <platform name="Gecko" version={Utils.appInfo.platformVersion} build={Utils.appInfo.platformBuildID}/>
+    <adblock-plus version={Utils.addonVersion} locale={Utils.appLocale}/>
+    <application name={Services.appinfo.name} vendor={Services.appinfo.vendor} version={Services.appinfo.version} userAgent={window.navigator.userAgent}/>
+    <platform name="Gecko" version={Services.appinfo.platformVersion} build={Services.appinfo.platformBuildID}/>
     <options>
       <option id="enabled">{Prefs.enabled}</option>
       <option id="objecttabs">{Prefs.frameobjects}</option>
       <option id="collapse">{!Prefs.fastcollapse}</option>
-      <option id="privateBrowsing">{Prefs.privateBrowsing}</option>
+      <option id="privateBrowsing">{PrivateBrowsing.enabled}</option>
       <option id="subscriptionsAutoUpdate">{Prefs.subscriptions_autoupdate}</option>
     </options>
     <window/>
@@ -37,16 +45,7 @@ let reportsListDataSource =
 
   collectData: function(wnd, windowURI, callback)
   {
-    let data = null;
-    try
-    {
-      data = JSON.parse(Prefs.recentReports);
-    }
-    catch (e)
-    {
-      Cu.reportError(e);
-    }
-
+    let data = Prefs.recentReports;
     if (data && "length" in data)
     {
       for (let i = 0; i < data.length; i++)
@@ -100,20 +99,13 @@ let reportsListDataSource =
   addReport: function(site, reportURL)
   {
     this.list.unshift({site: site, reportURL: reportURL, time: Date.now()});
-    try
-    {
-      Prefs.recentReports = JSON.stringify(this.list);
-    }
-    catch (e)
-    {
-      Cu.reportError(e);
-    }
+    Prefs.recentReports = this.list;
   },
 
   clear: function()
   {
     this.list = [];
-    Prefs.recentReports = JSON.stringify(this.list);
+    Prefs.recentReports = this.list;
     E("recentReports").hidden = true;
   },
 
@@ -122,7 +114,7 @@ let reportsListDataSource =
     if (event.button != 0 || !event.target || !event.target.hasAttribute("url"))
       return;
 
-    Utils.loadInBrowser(event.target.getAttribute("url"));
+    UI.loadInBrowser(event.target.getAttribute("url"));
   }
 };
 
@@ -163,7 +155,7 @@ let requestsDataSource =
       }
 
       // Location is meaningless for element hiding hits
-      if (entry.filter && entry.filter instanceof ElemHideFilter)
+      if (entry.filter && entry.filter instanceof ElemHideBase)
         delete requestXML.@location;  
         
       if (entry.filter)
@@ -551,6 +543,9 @@ let errorsDataSource =
 {
   collectData: function(wnd, windowURI, callback)
   {
+    let {addonID} = require("info");
+    addonID = addonID.replace(/[\{\}]/g, "");
+
     let messages = {};
     Cc["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService).getMessageArray(messages, {});
     messages = messages.value || [];
@@ -558,7 +553,8 @@ let errorsDataSource =
     {
       return (message instanceof Ci.nsIScriptError &&
           !/^https?:/i.test(message.sourceName) &&
-          (/adblock/i.test(message.errorMessage) || /adblock/i.test(message.sourceName)));
+          (/adblock/i.test(message.errorMessage) || /adblock/i.test(message.sourceName) ||
+           message.errorMessage.indexOf(addonID) >= 0 || message.sourceName.indexOf(addonID) >= 0));
     });
     if (messages.length > 10)   // Only the last 10 messages
       messages = messages.slice(messages.length - 10, messages.length);
@@ -571,7 +567,7 @@ let errorsDataSource =
       let [pathID, placeholder] = pathList[i];
       try
       {
-        let file = Utils.dirService.get(pathID, Ci.nsIFile);
+        let file = FileUtils.getDir(pathID, [], false);
         censored[file.path.replace(/[\\\/]+$/, '')] = placeholder;
         let uri = Utils.ioService.newFileURI(file);
         censored[uri.spec.replace(/[\\\/]+$/, '')] = placeholder;
@@ -614,18 +610,9 @@ let extensionsDataSource =
 
   collectData: function(wnd, windowURI, callback)
   {
-    let AddonManager = null;
     try
     {
-      let namespace = {};
-      Cu.import("resource://gre/modules/AddonManager.jsm", namespace);
-      AddonManager = namespace.AddonManager;
-    } catch (e) {}
-
-    if (AddonManager)
-    {
-      // Gecko 2.0
-      let me = this;
+      let AddonManager = Cu.import("resource://gre/modules/AddonManager.jsm", null).AddonManager;
       AddonManager.getAddonsByTypes(["extension", "plugin"], function(items)
       {
         for (let i = 0; i < items.length; i++)
@@ -633,38 +620,14 @@ let extensionsDataSource =
           let item = items[i];
           if (!item.isActive)
             continue;
-          me.data.appendChild(<extension id={item.id} name={item.name} type={item.type} version={item.version}/>);
+          this.data.appendChild(<extension id={item.id} name={item.name} type={item.type} version={item.version}/>);
         }
         callback();
-      });
+      }.bind(this));
     }
-    else if ("@mozilla.org/extensions/manager;1" in Cc)
+    catch (e)
     {
-      // Gecko 1.9.x
-      let extensionManager = Cc["@mozilla.org/extensions/manager;1"].getService(Ci.nsIExtensionManager);
-    	let ds = extensionManager.datasource;
-      let rdfService = Cc["@mozilla.org/rdf/rdf-service;1"].getService(Ci.nsIRDFService);
-      let list = {};
-      let items = extensionManager.getItemList(Ci.nsIUpdateItem.TYPE_EXTENSION | Ci.nsIUpdateItem.TYPE_PLUGIN, {});
-      for (let i = 0; i < items.length; i++)
-      {
-        let item = items[i];
-
-        // Check whether extension is disabled - yuk...
-        let source = rdfService.GetResource("urn:mozilla:item:" + item.id);
-        let link = rdfService.GetResource("http://www.mozilla.org/2004/em-rdf#isDisabled");
-        let target = ds.GetTarget(source, link, true);
-      	if (target instanceof Ci.nsIRDFLiteral && target.Value == "true")
-      		continue;
-
-        this.data.appendChild(<extension id={item.id} name={item.name} type={item.type == Ci.nsIUpdateItem.TYPE_EXTENSION ? "extension" : "plugin"} version={item.version}/>);
-      }
-      callback();
-    }
-    else
-    {
-      // No add-on manager, no extension manager - we must be running in K-Meleon.
-      // Skip this step.
+      // No add-on manager, what's going on? Skip this step.
       callback();
     }
   },
@@ -678,6 +641,151 @@ let extensionsDataSource =
   }
 };
 
+let subscriptionUpdateDataSource =
+{
+  contentWnd: null,
+  type: null,
+  outdated: null,
+  needUpdate: null,
+
+  collectData: function(wnd, windowURI, callback)
+  {
+    this.contentWnd = wnd;
+    let now = Date.now() / MILLISECONDS_IN_SECOND;
+    let outdatedThreshold = now - 14 * SECONDS_IN_DAY;
+    let needUpdateThreshold = now - 1 * SECONDS_IN_HOUR;
+
+    this.outdated = [];
+    this.needUpdate = [];
+
+    let subscriptions = FilterStorage.subscriptions.filter(issuesDataSource.subscriptionFilter);
+    for (let i = 0; i < subscriptions.length; i++)
+    {
+      let lastSuccess = subscriptions[i].lastSuccess;
+      if (lastSuccess < outdatedThreshold)
+        this.outdated.push(subscriptions[i]);
+      if (lastSuccess < needUpdateThreshold)
+        this.needUpdate.push(subscriptions[i]);
+    }
+
+    callback();
+  },
+
+  updatePage: function(type)
+  {
+    this.type = type;
+    E("updateInProgress").hidden = (type != "false positive" || this.needUpdate.length == 0);
+    E("outdatedSubscriptions").hidden = !E("updateInProgress").hidden || this.outdated.length == 0;
+    if (!E("outdatedSubscriptions").hidden)
+    {
+      let template = E("outdatedSubscriptionTemplate");
+      let list = E("outdatedSubscriptionsList");
+      while (list.lastChild)
+        list.removeChild(list.lastChild);
+
+      for (let i = 0; i < this.outdated.length; i++)
+      {
+        let subscription = this.outdated[i];
+        let entry = template.cloneNode(true);
+        entry.removeAttribute("id");
+        entry.removeAttribute("hidden");
+        entry.setAttribute("_url", subscription.url);
+        entry.setAttribute("tooltiptext", subscription.url);
+        entry.textContent = subscription.title;
+        list.appendChild(entry);
+      }
+    }
+    return !E("updateInProgress").hidden || !E("outdatedSubscriptions").hidden;
+  },
+
+  showPage: function()
+  {
+    document.documentElement.canAdvance = false;
+
+    if (!E("updateInProgress").hidden)
+    {
+      document.documentElement.canRewind = false;
+
+      for (let i = 0; i < this.needUpdate.length; i++)
+        Synchronizer.execute(this.needUpdate[i], true, true);
+
+      let listener = function(action)
+      {
+        if (!/^subscription\./.test(action))
+          return;
+
+        for (let i = 0; i < this.needUpdate.length; i++)
+          if (Synchronizer.isExecuting(this.needUpdate[i].url))
+            return;
+
+        FilterNotifier.removeListener(listener);
+        E("updateInProgress").hidden = "true";
+
+        let filtersRemoved = false;
+        let requests = requestsDataSource.origRequests;
+        for (let i = 0; i < requests.length; i++)
+          if (requests[i].filter && !requests[i].filter.subscriptions.filter(function(s) !s.disabled).length)
+            filtersRemoved = true;
+
+        if (filtersRemoved)
+        {
+          // Force the user to reload the page
+          E("updateFixedIssue").hidden = false;
+          document.documentElement.canAdvance = true;
+
+          let nextButton = document.documentElement.getButton("next");
+          nextButton.label = E("updatePage").getAttribute("reloadButtonLabel");
+          nextButton.accessKey = E("updatePage").getAttribute("reloadButtonAccesskey");
+          document.documentElement.addEventListener("wizardnext", function(event)
+          {
+            event.preventDefault();
+            event.stopPropagation();
+            window.close();
+            this.contentWnd.location.reload();
+          }.bind(this), true);
+        }
+        else
+        {
+          this.collectData(null, null, function() {});
+          this.needUpdate = [];
+          if (this.outdated.length)
+          {
+            document.documentElement.canRewind = true;
+
+            this.updatePage(this.type);
+            this.showPage();
+          }
+          else
+          {
+            // No more issues, make sure to remove this page from history and
+            // advance to the next page.
+            document.documentElement.canRewind = true;
+            document.documentElement.canAdvance = true;
+
+            let next = document.documentElement.currentPage.next;
+            document.documentElement.rewind();
+            document.documentElement.currentPage.next = next;
+
+            document.documentElement.advance();
+          }
+        }
+      }.bind(this);
+
+      FilterNotifier.addListener(listener);
+      window.addEventListener("unload", function()
+      {
+        FilterNotifier.removeListener(listener);
+      });
+    }
+  },
+
+  updateOutdated: function()
+  {
+    for (let i = 0; i < this.outdated.length; i++)
+      Synchronizer.execute(this.outdated[i], true, true);
+  }
+}
+
 let issuesDataSource =
 {
   contentWnd: null,
@@ -847,16 +955,23 @@ let issuesDataSource =
                                  !E("issuesNoSubscriptionsBox").hidden ||
                                  !E("issuesSubscriptionCountBox").hidden;
 
+    let page = E("typeSelectorPage");
+    if (subscriptionUpdateDataSource.updatePage(type))
+    {
+      page.next = "update";
+      page = E("updatePage");
+    }
+
     if (E("issuesWhitelistBox").hidden && E("issuesDisabledBox").hidden &&
         E("issuesNoFiltersBox").hidden && E("issuesNoSubscriptionsBox").hidden &&
         E("issuesOwnFiltersBox").hidden && E("issuesDisabledFiltersBox").hidden &&
         E("issuesDisabledSubscriptionsBox").hidden && E("issuesSubscriptionCountBox").hidden)
     {
-      E("typeSelectorPage").next = "screenshot";
+      page.next = "screenshot";
     }
     else
     {
-      E("typeSelectorPage").next = "issues";
+      page.next = "issues";
     }
   },
 
@@ -967,7 +1082,7 @@ let issuesDataSource =
 
 let dataCollectors = [reportsListDataSource, requestsDataSource, filtersDataSource, subscriptionsDataSource,
                       screenshotDataSource, framesDataSource, errorsDataSource, extensionsDataSource,
-                      issuesDataSource];
+                      subscriptionUpdateDataSource, issuesDataSource];
 
 //
 // Wizard logic
@@ -979,6 +1094,18 @@ function initWizard()
   E("typeGroup").selectedItem = null;
   document.documentElement.addEventListener("pageshow", updateNextButton, false);
 
+  // Move wizard header
+  let header = document.getAnonymousElementByAttribute(document.documentElement, "class", "wizard-header");
+  if (header)
+  {
+    document.getElementById("wizardHeaderLabel").setAttribute("value", document.documentElement.wizardPages[0].getAttribute("label"));
+    document.documentElement.insertBefore(document.getElementById("wizardHeader"), document.documentElement.firstChild);
+    document.documentElement.addEventListener("pageshow", function()
+    {
+      document.getElementById("wizardHeaderDeck").selectedIndex = (document.documentElement.pageIndex == 0 ? 0 : 1);
+    }, false);
+  }
+
   // Move privacy link
   let extraButton = document.documentElement.getButton("extra1");
   extraButton.parentNode.insertBefore(E("privacyLink"), extraButton);
@@ -1098,12 +1225,7 @@ function initTypeWarningPage()
 
   let template = textElement.textContent.replace(/[\r\n\s]+/g, " ");
 
-  let beforeLink, linkText, afterLink;
-  if (/(.*)\[link\](.*)\[\/link\](.*)/.test(template))
-    [beforeLink, linkText, afterLink] = [RegExp.$1, RegExp.$2, RegExp.$3];
-  else
-    [beforeLink, linkText, afterLink] = ["", template, ""];
-
+  let [, beforeLink, linkText, afterLink] = /(.*)\[link\](.*)\[\/link\](.*)/.exec(template) || [null, "", template, ""];
   while (textElement.firstChild && textElement.firstChild.nodeType != Node.ELEMENT_NODE)
     textElement.removeChild(textElement.firstChild);
   while (textElement.lastChild && textElement.lastChild.nodeType != Node.ELEMENT_NODE)
@@ -1266,15 +1388,10 @@ function reportSent(event)
     let errorElement = E("sendReportError");
     let template = errorElement.getAttribute("textTemplate").replace(/[\r\n\s]+/g, " ");
   
-    let beforeLink, linkText, afterLink;
-    if (/(.*)\[link\](.*)\[\/link\](.*)/.test(template))
-      [beforeLink, linkText, afterLink] = [RegExp.$1, RegExp.$2, RegExp.$3];
-    else
-      [beforeLink, linkText, afterLink] = ["", template, ""];
-
+    let [, beforeLink, linkText, afterLink] = /(.*)\[link\](.*)\[\/link\](.*)/.exec(template) || [null, "", template, ""];
     beforeLink = beforeLink.replace(/\?1\?/g, errorMessage);
     afterLink = afterLink.replace(/\?1\?/g, errorMessage);
-  
+
     while (errorElement.firstChild && errorElement.firstChild.nodeType != Node.ELEMENT_NODE)
       errorElement.removeChild(errorElement.firstChild);
     while (errorElement.lastChild && errorElement.lastChild.nodeType != Node.ELEMENT_NODE)
@@ -1311,7 +1428,7 @@ function reportSent(event)
       button.setAttribute("url", link);
       button.removeAttribute("disabled");
 
-      if (!Prefs.privateBrowsing)
+      if (!PrivateBrowsing.enabled)
         reportsListDataSource.addReport(framesDataSource.site, link);
     } catch (e) {}
     E("copyLinkBox").hidden = false;
@@ -1331,7 +1448,7 @@ function processLinkClick(event)
     link = link.parentNode;
 
   if (link && (link.protocol == "http:" || link.protocol == "https:"))
-    Utils.loadInBrowser(link.href);
+    UI.loadInBrowser(link.href);
 }
 
 function copyLink(url)
