@@ -22,10 +22,6 @@ var disabledMatcher = new CombinedMatcher();
 var docDomainThirdParty = null;
 var docDomainFirstParty = null;
 
-var abpHooks = null;
-
-let lastSelectionProp = "abpSelected" + RequestNotifier.getDataSeed();
-
 function init() {
   docDomainThirdParty = document.documentElement.getAttribute("docDomainThirdParty");
   docDomainFirstParty = document.documentElement.getAttribute("docDomainFirstParty");
@@ -46,16 +42,24 @@ function init() {
     E("searchField").focus();
 
   var selected = null;
-  if (/sidebarDetached\.xul$/.test(parent.location.href)) {
+  if (/sidebarDetached\.xul$/.test(parent.location.href))
+  {
     mainWin = parent.opener;
     mainWin.addEventListener("unload", mainUnload, false);
     E("detachButton").hidden = true;
     E("reattachButton").hidden = false;
-    if (!mainWin.document.getElementById("abp-sidebar"))
+
+    let mustDetach = parent.arguments[0];
+    if (mustDetach)
       E("reattachButton").setAttribute("disabled", "true");
-    if (mainWin.document.getElementById("abp-key-sidebar")) {
-      var sidebarKey = mainWin.document.getElementById("abp-key-sidebar").cloneNode(true);
-      parent.document.getElementById("detached-keyset").appendChild(parent.document.importNode(sidebarKey, true));
+    if ("sidebar" in UI.hotkeys)
+    {
+      let {KeySelector} = require("keySelector");
+      parent.addEventListener("keypress", function(event)
+      {
+        if (KeySelector.matchesKey(event, UI.hotkeys.sidebar))
+          doClose();
+      }, false);
     }
 
     // Set default size/position unless already persisted
@@ -69,8 +73,8 @@ function init() {
         wnd.setAttribute(attr, defaults[attr]);
   }
 
-  abpHooks = mainWin.document.getElementById("abp-hooks");
-  window.__defineGetter__("content", function() {return abpHooks.getBrowser().contentWindow;});
+  let {getBrowser, addBrowserLocationListener} = require("appSupport");
+  window.__defineGetter__("content", function() {return getBrowser(mainWin).contentWindow;});
 
   // Initialize matcher for disabled filters
   reloadDisabledFilters();
@@ -84,7 +88,8 @@ function init() {
   handleLocationChange();
 
   // Install a progress listener to catch location changes
-  abpHooks.getBrowser().addProgressListener(progressListener);
+  if (addBrowserLocationListener)
+    addBrowserLocationListener(mainWin, handleLocationChange, true);
 }
 
 // To be called for a detached window when the main window has been closed
@@ -100,7 +105,9 @@ function cleanUp() {
   Prefs.removeListener(onPrefChange);
   E("list").view = null;
 
-  abpHooks.getBrowser().removeProgressListener(progressListener);
+  let {removeBrowserLocationListener} = require("appSupport");
+  if (removeBrowserLocationListener)
+    removeBrowserLocationListener(mainWin, handleLocationChange);
   mainWin.removeEventListener("unload", mainUnload, false);
 }
 
@@ -162,7 +169,7 @@ function onSelectionChange() {
   if (item && window.content)
   {
     let key = item.location + " " + item.type + " " + item.docDomain;
-    window.content.document.setUserData(lastSelectionProp, key, null);
+    RequestNotifier.storeSelection(window.content, key);
     treeView.itemToSelect = null;
   }
 
@@ -176,7 +183,7 @@ function handleLocationChange()
     requestNotifier.shutdown();
 
   treeView.clearData();
-  treeView.itemToSelect = window.content.document.getUserData(lastSelectionProp);
+  treeView.itemToSelect = RequestNotifier.getSelection(window.content);
   requestNotifier = new RequestNotifier(window.content, function(wnd, node, item, scanComplete)
   {
     if (item)
@@ -201,6 +208,13 @@ function setMultilineContent(box, text, noRemove)
 
 // Fill in tooltip data before showing it
 function fillInTooltip(e) {
+  // Prevent tooltip from overlapping menu
+  if (E("context").state == "open")
+  {
+    e.preventDefault();
+    return;
+  }
+
   var item;
   if (treeView.data && !treeView.data.length)
     item = treeView.getDummyTooltip();
@@ -208,7 +222,10 @@ function fillInTooltip(e) {
     item = treeView.getItemAt(e.clientX, e.clientY);
 
   if (!item)
-    return false;
+  {
+    e.preventDefault();
+    return;
+  }
 
   let filter = ("filter" in item && item.filter ? item.filter : null);
   let size = ("tooltip" in item ? null : getItemSize(item));
@@ -290,8 +307,6 @@ function fillInTooltip(e) {
   }
   else
     E("tooltipPreviewBox").hidden = true;
-
-  return true;
 }
 
 const visual = {
@@ -327,7 +342,6 @@ function fillInContext(/**Event*/ e)
   {
     let filter = item.filter;
     let menuItem = E(filter.disabled ? "contextEnableFilter" : "contextDisableFilter");
-    menuItem.filter = filter;
     menuItem.setAttribute("label", menuItem.getAttribute("labeltempl").replace(/\?1\?/, filter.text));
     menuItem.hidden = false;
 
@@ -335,16 +349,12 @@ function fillInContext(/**Event*/ e)
     {
       let domain = null;
       try {
-        domain = content.location.host;
-        domain = Utils.effectiveTLD.getBaseDomainFromHost(domain);
+        domain = Utils.effectiveTLD.getBaseDomainFromHost(item.docDomain);
       } catch (e) {}
 
       if (domain && !filter.isActiveOnlyOnDomain(domain))
       {
         menuItem = E("contextDisableOnSite");
-        menuItem.item = item;
-        menuItem.filter = filter;
-        menuItem.domain = domain;
         menuItem.setAttribute("label", menuItem.getAttribute("labeltempl").replace(/\?1\?/, domain));
         menuItem.hidden = false;
       }
@@ -362,6 +372,22 @@ function fillInContext(/**Event*/ e)
   return true;
 }
 
+/**
+ * Resets context menu data once the context menu is closed.
+ */
+function clearContextMenu(/**Event*/ event)
+{
+  if (event.eventPhase != event.AT_TARGET)
+    return;
+
+  {
+    let menuItem = E("contextDisableOnSite");
+    menuItem.item = item;
+    menuItem.filter = filter;
+    menuItem.domain = domain;
+  }
+}
+
 /**
  * Processed mouse clicks on the item list.
  * @param {Event} event
@@ -403,7 +429,7 @@ function openInTab(item, /**Event*/ event)
   for each (let item in items)
   {
     if (item && item.typeDescr != "ELEMHIDE")
-      Utils.loadInBrowser(item.location, mainWin, event);
+      UI.loadInBrowser(item.location, mainWin, event);
   }
 }
 
@@ -434,7 +460,7 @@ function editFilter()
   if (!("location") in item)
     item.location = undefined
 
-  Utils.openFiltersDialog(item.filter);
+  UI.openFiltersDialog(item.filter);
 }
 
 function enableFilter(filter, enable) {
@@ -446,22 +472,37 @@ function enableFilter(filter, enable) {
 /**
  * Edits the filter to disable it on a particular domain.
  */
-function disableOnSite(item, /**Filter*/ filter, /**String*/ domain)
+function disableOnSite()
 {
+  let item = treeView.getSelectedItem();
+  let filter = item.filter;
+  if (!(filter instanceof ActiveFilter) || filter.disabled || !filter.subscriptions.length || filter.subscriptions.some(function(subscription) !(subscription instanceof SpecialSubscription)))
+    return;
+
+  let domain;
+  try {
+    domain = Utils.effectiveTLD.getBaseDomainFromHost(item.docDomain).toUpperCase();
+  }
+  catch (e)
+  {
+    return;
+  }
+
   // Generate text for new filter that excludes current domain
-  domain = domain.toUpperCase();
   let text = filter.text;
   if (filter instanceof RegExpFilter)
   {
-    if (Filter.optionsRegExp.test(text))
+    let match = Filter.optionsRegExp.exec(text);
+    if (match)
     {
       let found = false;
-      let options = RegExp.$1.toUpperCase().split(",");
+      let options = match[1].toUpperCase().split(",");
       for (let i = 0; i < options.length; i++)
       {
-        if (/^DOMAIN=(.*)/.test(options[i]))
+        let match = /^DOMAIN=(.*)/.exec(options[i]);
+        if (match)
         {
-          let domains = RegExp.$1.split("|").filter(function(d) d != domain && d != "~" + domain && (d.length <= domain.length || d.lastIndexOf("." + domain) != d.length - domain.length - 1));
+          let domains = match[1].split("|").filter(function(d) d != domain && d != "~" + domain && (d.length <= domain.length || d.lastIndexOf("." + domain) != d.length - domain.length - 1));
           domains.push("~" + domain);
           options[i] = "DOMAIN=" + domains.join("|");
           found = true;
@@ -476,12 +517,13 @@ function disableOnSite(item, /**Filter*/ filter, /**String*/ domain)
     else
       text += "$domain=~" + domain.toLowerCase();
   }
-  else if (filter instanceof ElemHideFilter)
+  else if (filter instanceof ElemHideBase)
   {
-    if (/^([^#]+)(#.*)/.test(text))
+    let match = /^([^#]+)(#.*)/.exec(text);
+    if (match)
     {
-      let selector = RegExp.$2;
-      let domains = RegExp.$1.toUpperCase().split(",").filter(function(d) d != domain && (d.length <= domain.length || d != "~" + domain && d.lastIndexOf("." + domain) != d.length - domain.length - 1));
+      let selector = match[2];
+      let domains = match[1].toUpperCase().split(",").filter(function(d) d != domain && (d.length <= domain.length || d != "~" + domain && d.lastIndexOf("." + domain) != d.length - domain.length - 1));
       domains.push("~" + domain);
       text = domains.join(",").toLowerCase() + selector;
     }
@@ -506,7 +548,9 @@ function disableOnSite(item, /**Filter*/ filter, /**String*/ domain)
   FilterStorage.removeFilter(filter);
 
   // Update display
-  item.filter = null;
+  for (let i = 0; i < treeView.allData.length; i++)
+    if (treeView.allData[i].filter == filter)
+      treeView.allData[i].filter = null;
   treeView.boxObject.invalidate();
 }
 
@@ -515,7 +559,7 @@ function copyToClipboard() {
   if (!items.length)
     return;
 
-  Utils.clipboardHelper.copyString(items.map(function(item) {return item.location}).join(Utils.getLineBreak()));
+  Utils.clipboardHelper.copyString(items.map(function(item) {return item.location}).join(IO.lineBreak));
 }
 
 function copyFilter() {
@@ -526,7 +570,7 @@ function copyFilter() {
   if (!items.length)
     return;
 
-  Utils.clipboardHelper.copyString(items.map(function(item) {return item.filter.text}).join(Utils.getLineBreak()));
+  Utils.clipboardHelper.copyString(items.map(function(item) {return item.filter.text}).join(IO.lineBreak));
 }
 
 function selectAll() {
@@ -676,16 +720,6 @@ function createSortWithFallback(cmpFunc, fallbackFunc, desc) {
   }
 }
 
-var progressListener =
-{
-  onLocationChange: function() handleLocationChange(),
-  onProgressChange: function() {},
-  onSecurityChange: function() {},
-  onStateChange: function() {},
-  onStatusChange: function() {},
-  QueryInterface: XPCOMUtils.generateQI([Ci.nsIWebProgressListener, Ci.nsISupportsWeakReference])
-};
-
 // Item list's tree view object
 var treeView = {
   //
@@ -714,7 +748,7 @@ var treeView = {
     this.boxObject = boxObject;
     this.itemsDummy = boxObject.treeBody.getAttribute("noitemslabel");
     this.whitelistDummy = boxObject.treeBody.getAttribute("whitelistedlabel");
-    var stringAtoms = ["col-address", "col-type", "col-filter", "col-state", "col-size", "col-docDomain", "col-filterSource", "state-regular", "state-filtered", "state-whitelisted", "state-hidden"];
+    var stringAtoms = ["col-address", "col-type", "col-filter", "col-state", "col-size", "col-docDomain", "col-filterSource", "state-regular", "state-filtered", "state-whitelisted", "state-hidden", "state-hiddenexception"];
     var boolAtoms = ["selected", "dummy", "filter-disabled"];
     var atomService = Cc["@mozilla.org/atom-service;1"].getService(Ci.nsIAtomService);
     this.atoms = {};
@@ -743,12 +777,13 @@ var treeView = {
     if (!this.sortColumn)
     {
       let defaultSort = E("list").getAttribute("defaultSort");
-      if (/^(\w+)\s+(ascending|descending)$/.test(defaultSort))
+      let match = /^(\w+)\s+(ascending|descending)$/.exec(defaultSort);
+      if (match)
       {
-        this.sortColumn = E(RegExp.$1);
+        this.sortColumn = E(match[1]);
         if (this.sortColumn)
         {
-          sortDir = RegExp.$2;
+          sortDir = match[2];
           this.sortColumn.setAttribute("sortDirection", sortDir);
         }
       }
@@ -843,6 +878,8 @@ var treeView = {
           state = "state-filtered";
         else if (filter instanceof ElemHideFilter)
           state = "state-hidden";
+        else if (filter instanceof ElemHideException)
+          state = "state-hiddenexception";
       }
     }
     else {
@@ -1074,7 +1111,7 @@ var treeView = {
 
     return (item.location.toLowerCase().indexOf(this.filter) >= 0 ||
             (item.filter && item.filter.text.toLowerCase().indexOf(this.filter) >= 0) ||
-            item.typeDescr.toLowerCase().indexOf(this.filter) >= 0 ||
+            item.typeDescr.toLowerCase().indexOf(this.filter.replace(/-/g, "_")) >= 0 ||
             item.localizedDescr.toLowerCase().indexOf(this.filter) >= 0 ||
             (item.docDomain && item.docDomain.toLowerCase().indexOf(this.filter) >= 0) ||
             (item.docDomain && item.thirdParty && docDomainThirdParty.toLowerCase().indexOf(this.filter) >= 0) ||
@@ -1143,8 +1180,7 @@ var treeView = {
 
     let col = {};
     this.boxObject.getCellAt(x, y, {}, col, {});
-    if (col.value)
-      return col.value.id;
+    return (col.value ? col.value.id : null);
   },
 
   getDummyTooltip: function() {
