@@ -4,6 +4,9 @@
  * http://mozilla.org/MPL/2.0/.
  */
 
+Cu.import("resource://gre/modules/FileUtils.jsm");
+Cu.import("resource://gre/modules/Services.jsm");
+
 let subscriptionListLoading = false;
 
 function init()
@@ -196,19 +199,14 @@ function setCustomSubscription(title, url, mainSubscriptionTitle, mainSubscripti
     messageElement.removeAttribute("invisible");
     addMainCheckbox.removeAttribute("invisible");
 
-    let beforeLink, afterLink;
-    if (/(.*)\?1\?(.*)/.test(messageElement.getAttribute("_textTemplate")))
-      [beforeLink, afterLink] = [RegExp.$1, RegExp.$2, RegExp.$3];
-    else
-      [beforeLink, afterLink] = [messageElement.getAttribute("_textTemplate"), ""];
-
+    let [, beforeLink, afterLink] = /(.*)\?1\?(.*)/.exec(messageElement.getAttribute("_textTemplate")) || [null, messageElement.getAttribute("_textTemplate"), ""];
     while (messageElement.firstChild)
       messageElement.removeChild(messageElement.firstChild);
     messageElement.appendChild(document.createTextNode(beforeLink));
     let link = document.createElement("label");
     link.className = "text-link";
     link.setAttribute("tooltiptext", mainSubscriptionURL);
-    link.addEventListener("click", function() Utils.loadInBrowser(mainSubscriptionURL), false);
+    link.addEventListener("click", function() UI.loadInBrowser(mainSubscriptionURL), false);
     link.textContent = mainSubscriptionTitle;
     messageElement.appendChild(link);
     messageElement.appendChild(document.createTextNode(afterLink));
@@ -233,9 +231,8 @@ function validateURL(url)
 
   // Is this a file path?
   try {
-    let file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
-    file.initWithPath(url);
-    return Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService).newFileURI(file).spec;
+    let file = new FileUtils.File(url);
+    return Services.io.newFileURI(file).spec;
   } catch (e) {}
 
   // Is this a valid URL?
