@@ -4,19 +4,21 @@
 # version 2.0 (the "License"). You can obtain a copy of the License at
 # http://mozilla.org/MPL/2.0/.
 
-import os, sys, re, subprocess, jinja2, buildtools, codecs, hashlib, base64, shutil
+import os, sys, re, subprocess, jinja2, buildtools, codecs, hashlib, base64, shutil, urllib, json
 from ConfigParser import SafeConfigParser
 from StringIO import StringIO
 from zipfile import ZipFile, ZIP_STORED, ZIP_DEFLATED
+import xml.dom.minidom as minidom
 import buildtools.localeTools as localeTools
 
 KNOWN_APPS = {
   'conkeror':   '{a79fe89b-6662-4ff4-8e88-09950ad4dfde}',
   'emusic':     'dlm@emusic.com',
   'fennec':     '{a23983c0-fd0e-11dc-95ff-0800200c9a66}',
+  'fennec2':    '{aa3c5121-dab2-40e2-81ca-7ea25febc110}',
   'firefox':    '{ec8030f7-c20a-464f-9b0e-13a3a9e97384}',
-  'midbrowser':   '{aa5ca914-c309-495d-91cf-3141bbb04115}',
-  'prism':    'prism@developer.mozilla.org',
+  'midbrowser': '{aa5ca914-c309-495d-91cf-3141bbb04115}',
+  'prism':      'prism@developer.mozilla.org',
   'seamonkey':  '{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}',
   'songbird':   'songbird@songbirdnest.com',
   'thunderbird':  '{3550f703-e582-4d05-9a08-453d09bdfdc6}',
@@ -47,17 +49,18 @@ def getChromeSubdirs(baseDir, locales):
   return result
 
 def getXPIFiles(baseDir):
-  return [os.path.join(baseDir, file) for file in ('components', 'modules', 'defaults', 'bootstrap.js', 'chrome.manifest', 'icon.png', 'icon64.png')]
-
-def getTestEnvFiles(baseDir):
-  return [os.path.join(baseDir, file) for file in ('components', 'defaults', 'bootstrap.js', 'chrome.manifest', 'icon.png', 'icon64.png')]
+  for file in ('components', 'modules', 'lib', 'resources', 'defaults', 'chrome.manifest', 'icon.png', 'icon64.png'):
+    yield os.path.join(baseDir, file)
+  for file in os.listdir(baseDir):
+    if file.endswith('.js') or file.endswith('.xml'):
+      yield os.path.join(baseDir, file)
 
 def getIgnoredFiles(params):
   result = ['.incomplete']
   if not params['limitMetadata']:
     result.append('meta.properties')
   if params['releaseBuild']:
-    result.append('TimeLine.jsm')
+    result.append('timeline.js')
   return result
 
 def isValidLocale(localesDir, dir, includeIncomplete=False):
@@ -77,8 +80,11 @@ def getLocales(baseDir, includeIncomplete=False):
   return locales
 
 def getBuildNum(baseDir):
-  (result, dummy) = subprocess.Popen(['hg', 'id', '-n'], stdout=subprocess.PIPE).communicate()
-  return re.sub(r'\W', '', result)
+  try:
+    (result, dummy) = subprocess.Popen(['hg', 'id', '-n'], stdout=subprocess.PIPE).communicate()
+    return re.sub(r'\W', '', result)
+  except Exception:
+    return '0'
 
 def readMetadata(baseDir):
   metadata = SafeConfigParser()
@@ -103,10 +109,10 @@ def processFile(path, data, params):
     replacement = '\n'.join(map(lambda locale: r'\1%s\2%s\3' % (locale, locale), params['locales']))
     data = re.sub(localesRegExp, replacement, data)
 
-  if params['releaseBuild'] and path.endswith('.jsm'):
+  if params['releaseBuild'] and path.endswith('.js'):
     # Remove timeline calls from release builds
     timelineRegExp1 = re.compile(r'^.*\b[tT]imeLine\.(\w+)\(.*', re.M)
-    timelineRegExp2 = re.compile(r'^.*Cu\.import\([^()]*\bTimeLine\.jsm\"\).*', re.M)
+    timelineRegExp2 = re.compile(r'^.*\brequire\(\"timeline\"\).*', re.M)
     data = re.sub(timelineRegExp1, '', data)
     data = re.sub(timelineRegExp2, '', data)
 
@@ -129,17 +135,38 @@ def readLocaleMetadata(baseDir, locales):
       result[locale] = {}
   return result
 
+def getContributors(baseDir, metadata):
+  main = []
+  additional = set()
+  if metadata.has_section('contributors'):
+    options = metadata.options('contributors')
+    options.sort()
+    for option in options:
+      value = metadata.get('contributors', option)
+      if re.search(r'\D', option):
+        match = re.search(r'^\s*(\S+)\s+//([^/\s]+)/@(\S+)\s*$', value)
+        if not match:
+          print >>sys.stderr, 'Warning: unrecognized contributor location "%s"\n' % value
+          continue
+        dom = minidom.parse(os.path.join(baseDir, match.group(1)))
+        tags = dom.getElementsByTagName(match.group(2))
+        for tag in tags:
+          if tag.hasAttribute(match.group(3)):
+            for name in re.split(r'\s*,\s*', tag.getAttribute(match.group(3))):
+              additional.add(name)
+      else:
+        main.append(value)
+  return main + sorted(additional, key=unicode.lower)
+
 def getTranslators(localeMetadata):
-  translators = {}
+  translators = set()
   for locale in localeMetadata.itervalues():
     if 'translator' in locale:
       for translator in locale['translator'].split(','):
         translator = translator.strip()
         if translator:
-          translators[translator] = True
-  result = translators.keys()
-  result.sort()
-  return result
+          translators.add(translator)
+  return sorted(translators, key=unicode.lower)
 
 def createManifest(baseDir, params):
   global KNOWN_APPS, defaultLocale
@@ -182,7 +209,7 @@ def fixupLocales(baseDir, files, params):
 
   for locale in params['locales']:
     for file in reference.iterkeys():
-      path = 'locale/%s/%s' % (locale, file)
+      path = 'chrome/locale/%s/%s' % (locale, file)
       if path in files:
         data = localeTools.parseString(files[path].decode('utf-8'), path)
         for key, value in reference[file].iteritems():
@@ -191,27 +218,66 @@ def fixupLocales(baseDir, files, params):
       else:
         files[path] = reference[file]['_origData'].encode('utf-8')
 
-def createChromeJar(baseDir, params, files=None):
-  if not files:
-    files = {}
-  for name, path in getChromeSubdirs(baseDir, params['locales']).iteritems():
-    if os.path.isdir(path):
-      readFile(files, params, path, name)
-  if not params['limitMetadata']:
-    fixupLocales(baseDir, files, params)
-
-  data = StringIO()
-  jar = ZipFile(data, 'w', ZIP_STORED)
-  for name, value in files.iteritems():
-    jar.writestr(name, value)
-  jar.close()
-  return data.getvalue()
-
 def readXPIFiles(baseDir, params, files):
   for path in getXPIFiles(baseDir):
     if os.path.exists(path):
       readFile(files, params, path, os.path.basename(path))
 
+def addMissingFiles(baseDir, params, files):
+  templateData = {
+    'hasChrome': False,
+    'hasChromeRequires': False,
+    'hasShutdownHandlers': False,
+    'hasVersionPref': False,
+    'chromeWindows': [],
+    'requires': {},
+    'metadata': params['metadata'],
+    'multicompartment': params['multicompartment'],
+    'applications': dict((v, k) for k, v in KNOWN_APPS.iteritems()),
+  }
+
+  def checkScript(name):
+    content = files[name]
+    for match in re.finditer(r'(?:^|\s)require\("([\w\-]+)"\)', content):
+      templateData['requires'][match.group(1)] = True
+      if name.startswith('chrome/content/'):
+        templateData['hasChromeRequires'] = True
+    if name == 'defaults/prefs.js':
+      if re.search(r'\.currentVersion"', content):
+        templateData['hasVersionPref'] = True
+    if not '/' in name or name.startswith('lib/'):
+      if re.search(r'(?:^|\s)onShutdown\.', content):
+        templateData['hasShutdownHandlers'] = True
+
+  for name, content in files.iteritems():
+    if name == 'chrome.manifest':
+      templateData['hasChrome'] = True
+    elif name.endswith('.js'):
+      checkScript(name)
+    elif name.endswith('.xul'):
+      match = re.search(r'<(?:window|dialog)\s[^>]*\bwindowtype="([^">]+)"', content)
+      if match:
+        templateData['chromeWindows'].append(match.group(1))
+
+  while True:
+    missing = []
+    for module in templateData['requires']:
+      moduleFile = 'lib/' + module + '.js'
+      if not moduleFile in files:
+        path = os.path.join(buildtools.__path__[0], moduleFile)
+        if os.path.exists(path):
+          missing.append((path, moduleFile))
+    if not len(missing):
+      break
+    for path, moduleFile in missing:
+      readFile(files, params, path, moduleFile)
+      checkScript(moduleFile)
+
+  env = jinja2.Environment(loader=jinja2.FileSystemLoader(buildtools.__path__[0]))
+  env.filters['json'] = json.dumps
+  template = env.get_template('bootstrap.js.tmpl')
+  files['bootstrap.js'] = processFile('bootstrap.js', template.render(templateData).encode('utf-8'), params)
+
 def signFiles(files, keyFile):
   import M2Crypto
   manifest = []
@@ -269,7 +335,7 @@ def writeXPI(files, outFile):
     zip.writestr(name, files[name])
   zip.close()
 
-def createBuild(baseDir, outFile=None, locales=None, buildNum=None, releaseBuild=False, keyFile=None, limitMetadata=False):
+def createBuild(baseDir, outFile=None, locales=None, buildNum=None, releaseBuild=False, keyFile=None, limitMetadata=False, multicompartment=False):
   if buildNum == None:
     buildNum = getBuildNum(baseDir)
   if locales == None:
@@ -290,6 +356,8 @@ def createBuild(baseDir, outFile=None, locales=None, buildNum=None, releaseBuild
   if outFile == None:
     outFile = getDefaultFileName(baseDir, metadata, version)
 
+  contributors = getContributors(baseDir, metadata)
+
   params = {
     'locales': locales,
     'releaseBuild': releaseBuild,
@@ -297,65 +365,24 @@ def createBuild(baseDir, outFile=None, locales=None, buildNum=None, releaseBuild
     'version': version.encode('utf-8'),
     'metadata': metadata,
     'limitMetadata': limitMetadata,
+    'contributors': contributors,
+    'multicompartment': multicompartment,
   }
   files = {}
   files['install.rdf'] = createManifest(baseDir, params)
-  if metadata.has_option('general', 'nojar'):
-    readFile(files, params, os.path.join(baseDir, 'chrome'), 'chrome')
-  else:
-    files['chrome/%s.jar' % metadata.get('general', 'baseName')] = createChromeJar(baseDir, params)
+  for name, path in getChromeSubdirs(baseDir, params['locales']).iteritems():
+    if os.path.isdir(path):
+      readFile(files, params, path, 'chrome/%s' % name)
+  if not params['limitMetadata']:
+    fixupLocales(baseDir, files, params)
   readXPIFiles(baseDir, params, files)
+  if not 'bootstrap.js' in files:
+    addMissingFiles(baseDir, params, files)
   if keyFile:
     signFiles(files, keyFile)
   writeXPI(files, outFile)
 
-def setupTestEnvironment(baseDir, profileDirs):
-  metadata = readMetadata(baseDir)
-  params = {
-    'locales': getLocales(baseDir, True),
-    'releaseBuild': True,
-    'buildNum': '',
-    'version': '99.9',
-    'metadata': metadata,
-    'limitMetadata': False,
-  }
-  files = {}
-  files['install.rdf'] = createManifest(baseDir, params)
-  for path in getTestEnvFiles(baseDir):
-    if os.path.exists(path):
-      readFile(files, params, path, os.path.basename(path))
-
-  if 'chrome.manifest' in files:
-    # Redirect manifest entries to the current directory
-    if sys.platform == 'win32':
-      import nturl2path
-      baseURL = 'file:' + nturl2path.pathname2url(os.path.abspath(baseDir))
-    else:
-      import urllib
-      baseURL = 'file://' + urllib.quote(os.path.abspath(baseDir))
-    files['chrome.manifest'] = re.sub(r'\bjar:chrome/\w+\.jar!', '%s/chrome' % baseURL, files['chrome.manifest'])
-    files['chrome.manifest'] = re.sub(r'(\s)chrome/', r'\1%s/chrome/' % baseURL, files['chrome.manifest'])
-    files['chrome.manifest'] = re.sub(r'\b(resource\s+\S+\s+)', r'\1%s/' % baseURL, files['chrome.manifest'])
-    files['chrome.manifest'] = re.sub(r'\b(content\s+\S+\s+)(\w+/)', r'\1%s/\2' % baseURL, files['chrome.manifest'])
-    if os.path.exists(os.path.join(baseDir, 'mochitest')):
-      files['chrome.manifest'] += 'content mochikit %s/mochitest/\n' % baseURL
-
-  id = metadata.get('general', 'id')
-  for dir in profileDirs:
-    # Remove packed XPI file if there is one
-    packedPath = os.path.join(dir, 'extensions', '%s.xpi' % id)
-    if os.path.isfile(packedPath):
-      os.remove(packedPath)
-
-    # Replace unpacked dir by new data
-    unpackedPath = os.path.join(dir, 'extensions', id)
-    if os.path.isdir(unpackedPath):
-      shutil.rmtree(unpackedPath)
-    for file, data in files.iteritems():
-      filePath = os.path.join(unpackedPath, *(file.split('/')))
-      parentDir = os.path.dirname(filePath)
-      if not os.path.exists(parentDir):
-        os.makedirs(parentDir)
-      handle = open(filePath, 'wb')
-      handle.write(data)
-      handle.close()
+def autoInstall(baseDir, host, port, multicompartment=False):
+  fileBuffer = StringIO()
+  createBuild(baseDir, outFile=fileBuffer, multicompartment=multicompartment)
+  urllib.urlopen('http://%s:%s/' % (host, port), data=fileBuffer.getvalue())
